/* Venux
 by ignoblegnome <http://letsmakerobots.com.com/user/5293> 
 Venux is a servo controlled venus fly trap fashioned out of brass.
 Based on the uServotino servo controller by Ro-Bot-X. <http://letsmakerobots.com/node/24522>

 Circuit:
  Simple Touch Sensor
  Two pn2222a NPN transistors in a Darlington configuration.
    Q1 emitter connected to Q2 base.
    Q2 emitter connected to ground.
    Collectors connected together.
    10k resistor pull-up from 5V connected to common collector.
    Common collector output to Digital Output 2 of Arduino (interrupt 0).
  Touch Contact1 connected to 5V.
  Touch Contact2 connected to Q1 Base via 100k resistor.
  
 Version 1.1: Add random behavior
 Version 1.0: Blinky swivels waist back and forth; reponds to touch by closing mouth.
 */

#define DEBUG 0    // set to 1 to print to serial monitor, 0 to disable
#include <Servo.h> 
 
Servo blinkywaist; // servo to turn blinky
Servo blinkymouth; // servo to control blinky's mouth
Servo pinkywaist; // servo to turn pinky
Servo pinkymouth; // servo to control pinky's mouth
Servo inkywaist; // servo to turn inky
Servo inkymouth; // servo to control inky's mouth
Servo clydewaist; // servo to turn clyde
Servo clydemouth; // servo to control clyde's mouth
                // a maximum of eight servo objects can be created 
                
// Constants
int blinkypin = 2; // digital pin 2 (interrupt 0)
int mouth_open = 120; // servo position for mouth open
int mouth_close = 60; // servo postion for mouth close
int waist_cw = 80;  // servo postion for waist clockwise
int waist_ccw = 120; // servo postion for waist counter clockwise

// Variables
int poswaist = 0;    // variable to store the servo position 
int waist1 = waist_cw;     // variable to store initial waist position 1
int waist2 = waist_ccw;    // variable to store initial waist position 2
int delayspeed = 100;    // variable to store initial delay speed.
int delaysleep = 100;    // variable to store pause between loops.

volatile int posmouth = mouth_open;    // variable to store the servo position 
volatile int blinkystat = 0;  // holds status of blinky's mouth (open=0; closed=1)
 
void setup() { 
  Serial.begin(9600); // start serial monitor

  randomSeed(analogRead(0));  // seed the random function before setting pullup resistors

  for (int pinindex = 4; pinindex < 14; pinindex++) {
     pinMode(pinindex, OUTPUT);  // set pins 4 to 13 as outputs
  }
  
  for (int pinindex = 14; pinindex < 20; pinindex++) {
     digitalWrite(pinindex, HIGH);  // set all the internal pullups for analog pins
  }
  
  digitalWrite(blinkypin, HIGH);
  attachInterrupt(0,close,FALLING);

  blinkywaist.attach(5); // servo attached to pin 5 will be positioned
  blinkywaist.write(90); // center the servo
  blinkymouth.attach(6); // servo attached to pin 6 will be positioned
  blinkymouth.write(mouth_open); // center the servo
  pinkywaist.attach(7); // servo attached to pin 7 will be positioned
  pinkywaist.write(90); // center the servo
  pinkymouth.attach(8); // servo attached to pin 8 will be positioned
  pinkymouth.write(90); // center the servo  
  inkywaist.attach(9); // servo attached to pin 9 will be positioned
  inkywaist.write(90); // center the servo
  inkymouth.attach(10); // servo attached to pin 10 will be positioned
  inkymouth.write(90); // center the servo  
  clydewaist.attach(11); // servo attached to pin 11 will be positioned
  clydewaist.write(90); // center the servo
  clydemouth.attach(12); // servo attached to pin 12 will be positioned
  clydemouth.write(90); // center the servo
} 
 
void loop() { 
  if(DEBUG){
    Serial.print("Open=1; Closed=0: ");
    Serial.println(blinkystat);
    Serial.print("Waist1: ");
    Serial.println(waist1);
    Serial.print("Waist2: ");
    Serial.println(waist2);
    Serial.print("Delay Speed: ");
    Serial.println(delayspeed);
    Serial.print("Delay Sleep: ");
    Serial.println(delaysleep);
    
  }
  digitalWrite(13, LOW);
  
  waist1 = random(80,101); // pick a number from 80 to 100
  waist2 = random(101,121); // pick a number from 101 to 120
  delayspeed = random(50,151); // pick a number from 50 to 150
  delaysleep = random(100,5001); //pick a number from 100 to 5000
  
  for(poswaist = waist1; poswaist < waist2; poswaist += 1)  // goes from 80 degrees to 120 degrees 
  {                                  // in steps of 1 degree 
    blinkywaist.write(poswaist);     // tell servo to go to position in variable 'poswaist' 
    blinkymouth.write(posmouth);     // we need to set the mouth postion too, so interrupt will work
    delay(delayspeed);                       // waits for the servo to reach the position 
  } 
  for(poswaist = waist2; poswaist>=waist1; poswaist -=1)     // goes from 120 degrees to 80 degrees 
  {                                
    blinkywaist.write(poswaist);     // tell servo to go to position in variable 'poswaist' 
    blinkymouth.write(posmouth);     // we need to set the mouth postion too, so interrupt will work
    delay(delayspeed);                       // waits for the servo to reach the position 
  }

//  delay(100);  // pause at far postion 
  
  posmouth = mouth_open;    // Open mouth again after interrupt
  
//  for(posmouth = mouth_open; posmouth>=mouth_close; posmouth -=1)     // goes from 130 degrees to 60 degrees 
//  {                                
//    blinkymouth.write(posmouth);     // tell servo to go to position in variable 'poswaist' 
//    delay(50);                       // waits 30ms for the servo to reach the position 
//  }
//  delay(100); // pause with mouth closed
  
//  for(posmouth = mouth_close; posmouth < mouth_open; posmouth += 1)  // goes from 0 degrees to 180 degrees 
//  {                                  // in steps of 1 degree 
//    blinkymouth.write(posmouth);              // tell servo to go to position in variable 'poswaist' 
//    delay(50);                       // waits 30ms for the servo to reach the position 

//  } 
  delay(delaysleep); // pause before cycle
} 

void close() {
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  // If interrupts come faster than 200ms, assume it's a bounce and ignore
  if (interrupt_time - last_interrupt_time > 200)
  {
     posmouth = mouth_close;
     blinkystat = 1;
     digitalWrite(13, HIGH);
  }
  last_interrupt_time = interrupt_time;
}
